package com.free.controller;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.free.config.CustomizationException;
import com.free.config.ExceptionEnum;
import com.free.config.ResponseFormat;
import com.free.dto.ChatRecordCreateDTO;
import com.free.dto.ChatRecordToReadDTO;
import com.free.entity.ChatRecord;
import com.free.entity.TransferApply;
import com.free.mq.MqService;
import com.free.service.ChatRecordService;
import com.free.service.TransferApplyService;
import com.free.utils.JwtUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping("/chatRecord")
@Api(tags = "对话记录")
public class ChatRecordController {
  @Autowired
  private ChatRecordService service;
  @Autowired
  private TransferApplyService transferApplyService;

  @Autowired
  private MqService mqService;

  /** 对话记录未读标识 */
  public static final String chatRecordNotReadSign = "0";

  /** 对话记录已读标识 */
  public static final String chatRecordReadSign = "1";

  
  @ApiOperation("修改已读")
  @PostMapping("/read")
  public Object read(@RequestBody @Valid ChatRecordToReadDTO body) {
    Map userInfo = null;
    try {
      userInfo = JwtUtil.getDetails(null);
      if (null == userInfo) {
        return ResponseFormat.error();
      }
    } catch (Exception e) {
      return ResponseFormat.error();
    }
    String user_id = (String) userInfo.get("id");
    QueryWrapper qw = new QueryWrapper<>();
    qw.eq("apply_id", body.getApply_id());
    qw.ne("speaker", user_id);
    ChatRecord updateData = new ChatRecord();
    updateData.setIs_read(chatRecordReadSign);
    service.update(updateData, qw);
    return ResponseFormat.success();
  }

  /** 创建数据 */
  
  @ApiOperation("创建数据")
  @PostMapping("")
  public Object save(@RequestBody @Valid ChatRecordCreateDTO body) {
    // 创建对话记录前,检查 转人工申请状态 及 发言人 是否在 转人工申请中;
    TransferApply apply = transferApplyService.getById(body.getApply_id());
    if (null == apply) {
      throw new CustomizationException(ExceptionEnum.APPLY_NOT_FOUNT);
    }
    String is_agree = apply.getIs_agree();
    if (!TransferApplyController.TransferApplyAgree.equals(is_agree)) {
      // 抛出异常,当前申请未受理
      throw new CustomizationException(ExceptionEnum.CHAT_APPLY_NOT_AGREE);
    }
    int is_close = apply.getIs_close();
    if (TransferApplyController.TransferApplyClose == is_close) {
      // 抛出异常,当前申请已结束
      throw new CustomizationException(ExceptionEnum.CHAT_APPLY_IS_CLOSE);
    }
    String customer_id = apply.getCustomer_id();
    String user_id = apply.getUser_id();
    String speaker = body.getSpeaker();
    if (customer_id.equals(speaker) && user_id.equals(speaker.toString())) {
      // 抛出异常,发言人既不是该申请的客服人员,也不是用户
      throw new CustomizationException(ExceptionEnum.CHAT_SPEAKER_NOT_IN_APPLY);
    }
    ChatRecord data = new ChatRecord();
    data.setTime(LocalDateTime.now());
    data.setApply_id(body.getApply_id());
    data.setSpeaker(body.getSpeaker());
    data.setContent(body.getContent());
    data.setContent_type(body.getContent_type());
    data.setUser_id(apply.getUser_id());
    data.setCustomer_id(apply.getCustomer_id());
    this.service.save(data);
    ChatRecord returnData = this.service.getById(data.getId());
    // 发送mq消息,往chat.${apply_id}中发,客服和用户都会订阅这个
    Map map = service.getUserAndCustomerNameByChatRecord(returnData);
    mqService.sendChatToApply(map);
    return ResponseFormat.success(map);
    // return ResponseFormat.success();
  }

  /** 列表查询 */
  
  @ApiOperation("查询数据列表")
  @SuppressWarnings({ "unchecked" })
  @GetMapping()
  public Object list(@RequestParam Map<String, Object> allParams) {
    Long skip = null, limit = null;
    Map map = new HashMap();
    QueryWrapper qw = new QueryWrapper<>();
    qw.orderByDesc("time");
    /** 参数处理处理 */
    for (String key : allParams.keySet()) {
      Object value = allParams.get(key);
      if (key.equals("skip")) {
        skip = Long.valueOf(String.valueOf(value));
      } else if (key.equals("limit")) {
        limit = Long.valueOf(String.valueOf(value));
      } else {
        // 其他为查询条件
        qw.eq(key, value);
      }
    }
    /** 分页处理 */
    List returnList = null;
    if (null != skip && null != limit) {
      IPage page = new Page<>(skip, limit);
      IPage pageResult = service.page(page, qw);
      List data = pageResult.getRecords();
      returnList = service.getUserAndCustomerName(data);
      long total = pageResult.getTotal();
      map.put("total", total);
    } else {
      List list = service.list(qw);
      returnList = service.getUserAndCustomerName(list);
    }
    map.put("data", returnList);

    return ResponseFormat.success(map);
  }

  /** 根据id查询 */
  @ApiOperation("查询数据")
  @GetMapping("/{id}")
  public Object fetch(@PathVariable String id) {
    Object newData = service.getById(id);
    return ResponseFormat.success(newData);
  }

  /** 根据id删除数据 , 无删除 */
  @ApiOperation("删除数据")
  @DeleteMapping("/{id}")
  public Object delete(@PathVariable String id) {
    QueryWrapper qw = new QueryWrapper<>();
    qw.eq("id", id);
    Long num = service.count(qw);
    if (num <= 0) {
      throw new CustomizationException(ExceptionEnum.NOT_FOUND);
    }
    service.removeById(id);
    return ResponseFormat.success();
  }
}
