package com.free.controller;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.free.annotation.PassToken;
import com.free.config.CustomizationException;
import com.free.config.ExceptionEnum;
import com.free.config.ResponseFormat;
import com.free.dto.LoginDTO;
import com.free.dto.PasswordUpdateDTO;
import com.free.dto.ResetPasswordDTO;
import com.free.entity.system.Admin;
import com.free.entity.system.Customer;
import com.free.entity.system.Role;
import com.free.service.system.AdminService;
import com.free.service.system.CustomerService;
import com.free.service.system.MenusService;
import com.free.service.system.RoleService;
import com.free.utils.BcryptUtil;
import com.free.utils.JwtUtil;
import com.free.utils.Utils;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping
@Api(tags = "登录服务")
public class LoginController {
  /** 超级管理员编码 */
  private String isSuperAdminCode = "0";
  /** 管理员登录类型 */
  public static String adminLoginType = "Admin";
  /** 客服人员登录类型 */
  public static String csLoginType = "Customer";

  @Autowired
  private AdminService adminService;

  @Autowired
  private CustomerService customerService;

  @Autowired
  private MenusService menusService;

  @Autowired
  private RoleService roleService;

  @PassToken
  @ApiOperation("客服服务登录")
  @PostMapping("/login/{type}")
  public Object login(@PathVariable String type, @RequestBody @Valid LoginDTO data) {
    Map<String, Object> userInfo = null;
    if (this.adminLoginType.equals(type)) {
      // 管理员
      QueryWrapper<Admin> qw = new QueryWrapper<>();
      qw.select("id", "nick_name", "account", "password", "role", "is_use", "is_super");
      qw.eq("account", data.getAccount());
      userInfo = this.adminService.getMap(qw);
    } else if (this.csLoginType.equals(type)) {
      // 客服人员
      QueryWrapper<Customer> qw = new QueryWrapper<>();
      qw.select("id", "nick_name", "account", "password", "role", "is_use");
      qw.eq("account", data.getAccount());
      userInfo = this.customerService.getMap(qw);
    } else {
      // 用户类型不对,抛出异常
      throw new CustomizationException(ExceptionEnum.USER_TYPE_ERROR);
    }
    // 未根据账号信息找到用户数据,抛出异常
    if (null == userInfo) {
      throw new CustomizationException(ExceptionEnum.ACCOUNT_NOT_FOUND);
    }
    // 密码验证
    String dbPwd = (String) userInfo.get("password");
    boolean isSame = BcryptUtil.matchesPassword(data.getPassword(), dbPwd);
    // 密码不一致,抛出异常
    if (!isSame) {
      throw new CustomizationException(ExceptionEnum.PASSWORD_ERROR);
    }
    userInfo.remove("password");
    // 将用户类型也放进去
    userInfo.put("type", type);
    // jwt加密,返回
    String code = JwtUtil.sign(userInfo);
    return ResponseFormat.success(code);

  }

  @ApiOperation("客服服务登出")
  @PostMapping("/logout")
  public Object logout() {
    // 获取token,然后注销
    String token = JwtUtil.getToken();
    JwtUtil.logOff(token);
    return ResponseFormat.success();
  }

  @ApiOperation("修改密码")
  @PostMapping("/pwd")
  public Object passwordUpdate(@RequestBody @Valid PasswordUpdateDTO data) {
    String type = data.getType();
    /** 加密后的密码 */
    String cpwd = BcryptUtil.encryptPassword(data.getPassword());
    if (null == cpwd) {
      throw new CustomizationException(ExceptionEnum.SERVICE_FAULT);
    }
    if (adminLoginType.equals(type)) {
      Admin adminData = new Admin();
      adminData.setId(data.getId());
      adminData.setPassword(cpwd);
      this.adminService.updateById(adminData);
    } else if (csLoginType.equals(type)) {
      // 客服人员修改密码
      Customer customerData = new Customer();
      customerData.setId(data.getId());
      customerData.setPassword(cpwd);
      this.customerService.updateById(customerData);
    } else {
      // 用户类型不对,抛出异常
      throw new CustomizationException(ExceptionEnum.USER_TYPE_ERROR);
    }

    return ResponseFormat.success();
  }

  @ApiOperation("重置密码")
  @PostMapping("/resetPwd")
  public Object resetPassword(@RequestBody @Valid ResetPasswordDTO data) {
    String type = data.getType();
    String password = Utils.randomString(6);
    String cpwd = BcryptUtil.encryptPassword(password);
    if (this.adminLoginType.equals(type)) {
      Admin adminData = new Admin();
      adminData.setId(data.getId());
      adminData.setPassword(cpwd);
      this.adminService.updateById(adminData);
    } else if (this.csLoginType.equals(type)) {
      Customer customerData = new Customer();
      customerData.setId(data.getId());
      customerData.setPassword(cpwd);
      this.customerService.updateById(customerData);
    } else {
      // 用户类型不对,抛出异常
      throw new CustomizationException(ExceptionEnum.USER_TYPE_ERROR);
    }
    Map map=new HashMap();
    map.put("password",password);
    return ResponseFormat.success(map);
  }

  /**
   * 根据token,查询出用户的权限,目录
   * 
   * @return
   */
  @GetMapping("/tokenView")
  public Object tokenView() {
    Map userMap = JwtUtil.getDetails(null);
    Map<String, Object> returnData = new HashMap<>();
    /**
     * 1.用户信息;
     * 2.用户类型;
     * 3-1.管理员用户:
     * 4.查看是否是超级管理员(is_super = 0):
     * 5-1: 是超级管理员 - 递归获取所有目录,全部显示
     * 5-2: 不是超级管理员 - 根据设置角色 获取指定目录
     * 3-2.客服人员:
     * 4.根据角色获取指定目录
     */
    String user_id = (String) userMap.get("id");
    String type = (String) userMap.get("type");
    Map<String, Object> userInfo = null;
    if (this.adminLoginType.equals(type)) {
      // 判断是否是超级管理员
      String is_super = (String) userMap.get("is_super");
      if (this.isSuperAdminCode.equals(is_super)) {
        List menus = this.menusService.recursionMenus();
        returnData.put("menus", menus);
        return ResponseFormat.success(returnData);
      }
      // 不是超级管理员,那也是管理员,查出管理员信息,之后一起处理
      QueryWrapper qw = new QueryWrapper<>();
      qw.eq("id", user_id);
      qw.eq("is_use", "0");
      userInfo = this.adminService.getMap(qw);
    } else if (this.csLoginType.equals(type)) {
      // 查客服人员最新信息
      QueryWrapper qw = new QueryWrapper<>();
      qw.eq("id", user_id);
      qw.eq("is_use", "0");
      userInfo = this.customerService.getMap(qw);
    }
    if (null == userInfo) {
      // 没有用户说明: 1.token有问题;2.用户数据有问题
      throw new CustomizationException(ExceptionEnum.TOKEN_ERROR);
    }
    // 根据角色获取目录
    String role = (String) userInfo.get("role");
    QueryWrapper roleQW = new QueryWrapper<>();
    roleQW.eq("id", role);
    roleQW.eq("is_use", "0");
    Role userRole = this.roleService.getOne(roleQW);
    if (null == userRole) {
      throw new CustomizationException(ExceptionEnum.USER_ROLE_NOT_FOUND);
    }
    ArrayNode menus = userRole.getMenu();
    // 根据前端选择的route_name进行目录的查询
    List menusRouteNameList = new ArrayList();
    for (int i = 0; i < menus.size(); i++) {
      JsonNode e = menus.get(i);
      String route_name = e.textValue();
      menusRouteNameList.add(route_name);
    }
    QueryWrapper menuQW = new QueryWrapper<>();
    menuQW.in("route_name", menusRouteNameList);
    menuQW.eq("is_use", "0");
    List menusList = this.menusService.list(menuQW);
    List returnMenus = this.menusService.recursionMenus(menusList);
    returnData.put("menus", returnMenus);
    return ResponseFormat.success(returnData);
  }
}
